@@ -1,10 +1,10 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2018, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2019, 2020 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;; Copyright © 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018, 2019, 2020 Paul Hoogendijk <paul@dezyne.org>
-;;; Copyright © 2018, 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -63,10 +63,18 @@
             makreel:unticked-dotted-name
             root->))
 
-(define %model-name (make-parameter #f))
 (define %id-alist (make-parameter #f))
+(define %model-name (make-parameter #f))
 (define %next-alist (make-parameter #f))
 
+(define-ast <port-pair> (<ast>)
+  (port)
+  (other))
+
+(define-method (.port.name (o <port-pair>)) (.name (.port o)))
+(define-method (.other.name (o <port-pair>)) (.name (.other o)))
+
+
 ;;;
 ;;; Ticked root.
 ;;;
@@ -156,6 +164,20 @@
 			    (mcrl2:process-continuation parent))))
       (_ (mcrl2:process-continuation parent)))))
 
+(define (last-statement? o)
+  (let* ((p (parent o <scope>))
+         (statements (ast:statement* p)))
+    (ast:eq? (last statements) o)))
+
+(define-method (makreel:assign-call-parameter (o <variable>))
+  (if (last-statement? o) '()
+      o))
+
+(define-method (makreel:assign-call-parameter (o <assign>))
+  (if (and (last-statement? o)
+           (ast:eq? (parent (.variable o) <scope>) (parent o <scope>))) '()
+           o))
+
 ;; FIXME: non-compatible copy from mcrl2 scope vs model ticking:
 ;;  <scope-name (IConsole) State'> vs <interface IConsole'>
 ;; implications for trace format mcrl2, templates
@@ -280,7 +302,7 @@
    (or (.port o) (parent o <on>)
        (let ((model (parent o <model>)))
          (if (is-a? model <interface>) model
-             ((compose .type car ast:provides-ports) o))))))
+             ((compose .type car ast:provides-port*) o))))))
 
 (define-method (makreel:model-name (o <model>))
   (string-join (ast:full-name o) ""))
@@ -291,32 +313,28 @@
 (define-method (makreel:scope-name (o <ast>))
   (and=> (parent o <model>) makreel:model-name))
 
+(define-method (makreel:multiple-provides? (o <port>))
+  (if (< 1 (length (ast:provides-port* o))) o
+      '()))
+
+(define-method (makreel:negate-multiple-provides? (o <port>))
+  (if (not (< 1 (length (ast:provides-port* o)))) o
+      '()))
+
 (define-method (ast:provides-interfaces (o <component>))
   (delete-duplicates (map .type (filter ast:provides? (ast:port* o)))))
 
 (define-method (makreel:flush-provides-ports (o <port>))
-  (ast:provides-ports (parent o <component>)))
+  (ast:provides-port* (parent o <component>)))
 
-(define-method (ast:provides-ports (o <component>))
-  (filter ast:provides? (ast:port* o)))
-
-(define-method (ast:requires-ports (o <component>))
-  (filter ast:requires? (ast:port* o)))
-
-(define-method (ast:requires+async-ports (o <component>))
-  (append (ast:requires-ports o) (ast:async-ports o)))
-
-(define-method (ast:async-ports (o <component>))
-  ((compose ast:port* .behavior) o))
-
-(define-method (ast:async-ports? (o <component>))
-  (if (pair? (ast:async-ports o)) o
+(define-method (ast:async-port*? (o <component>))
+  (if (pair? (ast:async-port* o)) o
       '()))
 
-(define-method (ast:non-external-ports (o <component>))
+(define-method (ast:non-external-port* (o <component>))
   (filter (negate ast:external?) (filter ast:requires? (ast:port* o))))
 
-(define-method (ast:external-ports (o <component>))
+(define-method (ast:external-port* (o <component>))
   (filter ast:external? (ast:port* o)))
 
 (define-method (makreel:action-sort (o <component>))
@@ -372,6 +390,9 @@
       (($ <void>) o)
       (_ type))))
 
+(define-method (makreel:type-constructor (o <port-pair>))
+  (makreel:type-constructor (.port o)))
+
 (define-method (makreel:modeling-sort (o <component>))
   (ast:interface* o))
 
@@ -480,10 +501,10 @@
   (or (and (not (find ast:requires? (ast:port* o))) o) '()))
 
 (define-method (ast:have-requires+async? (o <component>))
-  (if (pair? (ast:requires+async-ports o)) o '()))
+  (if (pair? (ast:requires+async-port* o)) o '()))
 
 (define-method (ast:have-no-requires+async? (o <component>))
-  (if (pair? (ast:requires+async-ports o)) '() o))
+  (if (pair? (ast:requires+async-port* o)) '() o))
 
 (define-method (makreel:queue-length (o <component>))
   (%queue-size))
@@ -526,7 +547,7 @@
         (if (is-a? model <interface>) model
             (let ((trigger (and=> (parent o <on>) (compose car ast:trigger*))))
               (if (and trigger (ast:provides? trigger)) ((compose .port) trigger)
-                  ((compose car ast:provides-ports) model)))))))
+                  ((compose car ast:provides-port*) model)))))))
 
 (define-method (makreel:event-prefix (o <port>))
   o)
@@ -542,15 +563,8 @@
   (makreel:event-prefix (parent o <on>)))
 
 (define-method (makreel:trigger-name (o <on>))
-  (let* ((trigger ((compose car ast:trigger*) o))
-         (event (.event trigger)))
-    (if (and (is-a? event <modeling-event>) (.silent? o))
-        (clone (make <silent-trigger>
-                 #:port.name (.port.name trigger)
-                 #:event.name (.event.name trigger)
-                 #:formals (.formals trigger))
-               #:parent (.parent trigger))
-       event)))
+  (let ((trigger ((compose car ast:trigger*) o)))
+    (.event trigger)))
 
 (define-method (makreel:interface-proc (o <interface>))
   o)
@@ -567,10 +581,6 @@
 (define-method (is-optional? (o <trigger>))
   (equal? "optional" (.event.name o)))
 
-(define-method (makreel:silent (o <the-end>))
-  (if (.silent? (parent o <on>)) o
-      '()))
-
 (define-method (makreel:proc (o <model>))
   (makreel:proc-list o))
 
@@ -601,7 +611,6 @@
 
 (define (unspecified? x) (eq? x *unspecified*))
 
-
 (define-method (makreel:on-proc (o <on>))
   (.statement o))
 
@@ -789,7 +798,24 @@
   (ast:variable* (.type o)))
 
 (define-method (makreel:provides-proc (o <component>))
-  (ast:provides-ports o))
+  (ast:provides-port* o))
+
+(define-method (makreel:provides-pair* (o <port>))
+  (map (cute make <port-pair> #:port o #:other <>) (ast:provides-port* o)))
+
+(define-method (makreel:provides-reply (o <port-pair>))
+  (let ((other (.other o)))
+    (if (ast:eq? other (.port o)) "r"
+        other)))
+
+(define-method (makreel:provides-reply (o <port>))
+  (ast:provides-port* o))
+
+(define-method (makreel:provides-reset-reply (o <port-pair>))
+  (let ((port (.port o))
+        (other (.other o)))
+    (if (ast:eq? other port) (format #f "~anil" (makreel:interface-name port))
+        other)))
 
 (define-method (makreel:rename-flush-provides (o <port>))
   (if (ast:provides? o) o
@@ -808,27 +834,19 @@
 (define-method (makreel:action-proc (o <model>))
   o)
 
+(define-method (makreel:switch-context (o <action>))
+  (let* ((component (parent o <component>))
+         (blocking? (and component
+                         (ast:requires? o)
+                         (find ast:blocking? (ast:requires-port* component)))))
+    (if (or (not blocking?) (not (ast:blocking? o))) '()
+        o)))
+
 (define-method (ast:trigger* (o <model>)) ;; FIXME: maybe use ast:in-triggers
   (delete-duplicates (tree-collect-filter (is? <declarative>) (is? <trigger>) o)
                      (lambda (a b) (and (equal? (.port.name a) (.port.name b))
                                         (equal? (.event.name a) (.event.name b))))))
 
-(define-method (ast:blocking? (o <port>))
-  (if (pair? (tree-collect-filter (negate (disjoin (is? <imperative>) (is? <expression>) (is? <location>))) (is? <blocking>) (parent o <model>))) o ;; FIXME: specify per port
-      '()))
-
-(define-method (ast:blocking? (o <component>))
-  (if (pair? (tree-collect-filter (negate (disjoin (is? <imperative>) (is? <expression>) (is? <location>))) (is? <blocking>) (is? <blocking>) o)) o
-      '()))
-
-(define-method (ast:provides-blocking? (o <component>))
-  (if (and (pair? (ast:requires-ports o))
-           (pair? (tree-collect-filter (negate (disjoin (is? <imperative>) (is? <expression>) (is? <location>))) (is? <blocking>) o))) (ast:provides-ports o)
-      '()))
-
-(define-method (ast:requires-blocking? (o <port>))
-  (ast:requires-ports (parent o <model>)))
-
 (define-method (ast:port-type-name (o <reply>)) ;; FIXME: return AST (interface/port) rather than string
   (let ((interface
             (or (and=> (.port o) .type)
@@ -836,7 +854,7 @@
                   (if (is-a? model <interface>) model
                       (let ((trigger (and=> (parent o <on>) (compose car ast:trigger*))))
                         (if (and trigger (ast:provides? trigger)) ((compose .type .port) trigger)
-                            ((compose .type car ast:provides-ports) model))))))))
+                            ((compose .type car ast:provides-port*) model))))))))
     (makreel:model-name interface)))
 
 (define-method (ast:port-type-name (o <assign>))
@@ -949,7 +967,7 @@
 ;;;
 (define (root-> o)
   (let ((queue-size (or (%queue-size)
-                        (command-line:get 'queue-size 3)))
+                        (command-line:get-number 'queue-size 3)))
         (init (command-line:get 'init)))
     (parameterize ((%queue-size queue-size)
                    (%id-alist '())
@@ -961,4 +979,5 @@
 
 (define* (ast-> ast #:key dir model)
   (let ((root (makreel:om ast)))
-    (root-> root)))
+    (if model (makreel:model->makreel root (makreel:get-model root model))
+        (root-> root))))
