@@ -1,6 +1,6 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2017, 2018, 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2019, 202, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2014, 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2014, 2018, 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
@@ -221,8 +221,7 @@ using CONTENT-ALIST to transform locations."
          (let* ((imports (append-map (match-lambda
                                        ((file-name . root) (.elements root)))
                                      imports))
-                (elements (append imports (.elements root)))
-                (elements (filter (negate (is? <import>)) elements)))
+                (elements (append imports (.elements root))))
            (clone root #:elements elements))))))
 
   (let* ((ast-alist
