@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2020, 2021 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -87,21 +87,51 @@
 
 (define-method (begin-step (pc <program-counter>) (instance <runtime:instance>) (trigger <trigger>))
   (%debug "* ~s ~s ~a\n" (name instance) (trigger->string trigger) "<begin-step>")
+  (define (trigger-runtime-port pc)
+    (runtime:port (.instance pc) (.port (.trigger pc))))
   (let* ((pc (push-pc pc trigger instance))
-         (pc (set-handling? pc #t))
          (port (.port trigger))
-         (pc (if (and port (ast:provides? port)) (set-reply pc #f)
+         (instance (.instance pc))
+         (port-name (or (.port.name trigger)
+                        (.name (.ast instance))))
+         (pc (if (and port (ast:provides? port)) (reset-reply pc port-name)
                  pc))
          (r:port (and port (runtime:port instance port))))
-    (if (not (assoc r:port (.blocked pc))) pc
+    (cond
+     ((and (is-a? instance <runtime:component>)
+           (or (get-handling pc instance)
+               (assq r:port (.blocked pc)))
+           (blocked-port pc instance)
+           (not (blocked-on-boundary? pc)))
+      (%debug "<blocked-error> collateral ~a ~a\n" (name instance) (trigger->string trigger))
+      (let ((message (format #f "port `~a' is collaterally blocked"
+                             (.name port))))
         (clone pc #:status (make <blocked-error>
                              #:ast (.statement pc)
-                             #:message (format #f "port `~a' is blocked" (.name port)))))))
+                             #:message message))))
+     ((let* ((r:other-port (runtime:other-port r:port))
+             (blocked (append (.blocked pc) (.collateral pc)))
+             (ports (map (compose runtime:other-port
+                                  trigger-runtime-port
+                                  rtc-block-pc cdr)
+                         blocked)))
+        (or (assq r:port (.blocked pc))
+            (and (ast:provides? trigger)
+                 (member r:other-port ports ast:equal?))))
+      (%debug "<blocked-error> blocked ~a ~a\n" (name instance) (trigger->string trigger))
+      (let ((message (format #f "port `~a' is blocked" (.name port))))
+        (clone pc #:status (make <blocked-error>
+                             #:ast (.statement pc)
+                             #:message message))))
+     (else
+      (let ((pc (set-handling! pc)))
+        pc)))))
 
 (define-method (begin-step (pc <program-counter>) (instance <runtime:port>) (trigger <trigger>))
   (%debug "* ~s ~s ~a\n" (name instance) (trigger->string trigger) "<begin-step>")
   (let* ((pc (push-pc pc trigger instance))
-         (pc (set-reply pc #f)))
+         (port-name (.name (.ast instance)))
+         (pc (reset-reply pc port-name)))
     pc))
 
 
@@ -133,11 +163,14 @@
 
 (define (mark-liveness? pc statement)
   (and (%liveness?)
-       (is-a? (.instance pc) <runtime:component>)
+       (or (and (eq? (%liveness?) 'component)
+                (is-a? (.instance pc) <runtime:component>))
+           (and (eq? (%liveness?) 'port)
+                (is-a? (.instance pc) <runtime:port>)))
        (not (is-a? statement <compound>))
        (not (is-a? statement <illegal>))
        (ast:imperative? statement)
-       (let* ((pc (clone pc #:status (make <end-of-trail>))))
+       (let ((pc (clone pc #:status (make <end-of-trail>))))
          (list pc))))
 
 (define-method (step (pc <program-counter>) (o <guard>))
@@ -162,7 +195,8 @@
 
 (define-method (step (pc <program-counter>) (o <illegal>))
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
-  (list (clone pc #:previous #f #:status (make <illegal-error> #:message "illegal" #:ast o))))
+  (let ((error (make <illegal-error> #:message "illegal" #:ast o)))
+    (list (clone pc #:status error))))
 
 (define-method (step (pc <program-counter>) (o <compound>))
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
@@ -215,30 +249,47 @@
          (r:port (runtime:port instance port))
          (other-instance other-port (runtime:other-instance+port instance r:port))
          (trigger (action->trigger other-port o))
+         (orig-pc pc)
          (pc (continuation pc o)))
     (cond
-     ((is-a? other-instance <runtime:component>)
-      (list (begin-step pc other-instance trigger)))
-     ((runtime:boundary-port? other-port)
-      (let* ((silent-traces ((@ (dzn vm run) run-silent) pc other-instance))
+     ((and (runtime:boundary-port? other-port)
+           (ast:requires? other-port)
+           (ast:external? other-port))
+      (let* ((run-external-modeling (@ (dzn vm run) run-external-modeling))
+             (silent-traces (run-external-modeling pc other-instance))
              (silent-pcs (map car silent-traces))
              (pcs (cons pc silent-pcs)))
         (map (cute begin-step <> other-instance trigger) pcs)))
-     (ast:injected? port
+     ((runtime:boundary-port? other-port)
+      (list (begin-step pc other-instance trigger)))
+     ((and (is-a? other-instance <runtime:component>)
+           (or (get-handling pc other-instance)
+               (or (assoc-ref (.blocked pc) other-port)
+                   (assoc-ref (.blocked pc) r:port)))
+           (or (blocked-port pc other-instance)
+               (and=> (.previous pc) blocked-on-boundary?)))
+      (let ((pc (collateral-block orig-pc other-instance)))
+        (list pc)))
+     ((is-a? other-instance <runtime:component>)
+      (list (begin-step pc other-instance trigger)))
+     (else
       (list pc)))))
 
 (define (step-async-action-down pc o)
   (match (.event.name o)
     ("req"
      (let* ((instance (.instance pc))
-            (trigger (clone (make <q-trigger> #:port.name (.port.name o) #:event.name "ack")
-                            #:parent ((compose .behavior .type .ast) instance)))
-            (ack (lambda (pc) (list (begin-step pc instance trigger))))
+            (q-trigger (make <q-trigger> #:port.name (.port.name o) #:event.name "ack"
+                             #:modeling? (ast:modeling? (.trigger pc))))
+            (q-trigger (clone q-trigger
+                              #:parent ((compose .behavior .type .ast) instance)))
+            (ack (lambda (pc) (list (begin-step pc instance q-trigger))))
             (rank (.rank instance))
             (r:port (runtime:port instance (.port o)))
             (req-pending? (find (compose (cute ast:eq? r:port <>) cadr) (.async pc))))
        (if req-pending?
-           (list (clone pc #:status (make <illegal-error> #:message "illegal" #:ast o)))
+           (let ((error (make <illegal-error> #:message "illegal" #:ast o)))
+             (list (clone pc #:status error)))
            (let ((pc (clone pc #:async (acons rank (cons r:port ack) (.async pc)))))
              (list (continuation pc o))))))
     ("clr"
@@ -257,14 +308,19 @@
          (port (.port o))
          (r:port (runtime:port instance (.port o)))
          (other-instance other-port (runtime:other-instance+port instance r:port))
+         (orig-pc pc)
          (pc (continuation pc o)))
 
     (define (q-trigger)
       (let* ((port-name (.name (.ast other-port)))
+             (trigger (.trigger pc))
              (q-trigger (make <q-trigger>
-                         #:event.name (.event.name o)
-                         #:port.name port-name
-                         #:location (.location o))))
+                          #:event.name (.event.name o)
+                          #:port.name port-name
+                          #:modeling? (or (ast:modeling? trigger)
+                                          (and=> (as trigger <q-trigger>)
+                                                 .modeling?))
+                          #:location (.location o))))
         (clone q-trigger #:parent (.type (.ast other-instance)))))
 
     (cond
@@ -302,27 +358,55 @@
 
 (define-method (step (pc <program-counter>) (o <reply>))
   (%debug "  ~s ~s ~a => ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o) (.expression o))
-  (let* ((value (get-reply pc))
-         (pc (if value
-                 (let ((error (make <second-reply-error> #:ast o #:previous (.parent value) #:message "second-reply")))
-                   (%debug "second reply, previous=~a\n" ((@@ (dzn vm report) step->location) value))
-                   (clone pc #:status error))
-                 (let ((value (eval-expression pc (.expression o))))
-                   (continuation (set-reply pc value) o))))
-         (reply-port (.port o)))
-    (cond ((ast:modeling? (.trigger pc))
-           (let ((pc (clone pc #:status (make <deadlock-error> #:ast o #:message "deadlock"))))
-             (list pc)))
-          ((let* ((trigger (.trigger pc))
-                  (trigger-port (.port trigger))
-                  (blocking? (parent o <blocking>))
+  (let* ((instance (.instance pc))
+         (port-name (or (.port.name o)
+                        (.port.name (.trigger pc))
+                        (.name (.ast instance))))
+         (previous (get-reply pc port-name))
+         (value (eval-expression pc (.expression o)))
+         (pc (if (not previous) (continuation (set-reply pc port-name value) o)
+                 (let ((error (make <second-reply-error>
+                                #:ast o
+                                #:previous (.parent previous)
+                                #:message "second-reply")))
+                   (%debug "second reply, previous=~a\n" (ast:location->string previous))
+                   (clone pc #:status error))))
+         (reply-port (.port o))
+         (trigger (.trigger pc)))
+    (cond ((let* ((trigger-port (.port trigger))
+                  (stack (pc->stack pc))
+                  (instance-stack (filter
+                                   (compose (cute eq? <> instance) .instance)
+                                   stack))
+                  (statements (map .statement instance-stack))
+                  (blocking? (or (pair? (.blocked pc))
+                                 (any (cute parent <> <blocking>) statements)))
                   (ports-eq? (ast:eq? reply-port trigger-port)))
              (and reply-port
+                  (or blocking?
+                      (and (or (ast:eq? (.port (rtc-trigger pc)) reply-port)
+                               (not (and=> (as trigger <q-trigger>) .modeling?)))
+                           (or (ast:provides? (rtc-trigger pc))
+                               (not (is-a? (ast:type value) <void>)))))
                   (or (ast:requires? trigger)
-                      (and ports-eq? blocking?)
-                      (and (not ports-eq?) (not blocking?)))))
-           (let* ((r:port (runtime:port (.instance pc) reply-port))
-                  (pc (clone pc #:released r:port)))
+                      (not ports-eq?)
+                      (and ports-eq? blocking?))))
+           (let* ((r:port (runtime:port instance reply-port))
+                  (released (.released pc))
+                  (released (if (memq r:port released) released
+                                (append released (list r:port))))
+                  (pc (clone pc #:released released)))
+             (list pc)))
+          ((or (ast:modeling? trigger)
+               (and=> (as trigger <q-trigger>) .modeling?)
+               (and reply-port
+                    (null? (.blocked pc))
+                    (not (ast:provides? (rtc-trigger pc)))
+                    (is-a? (ast:type value) <void>)
+                    (or (ast:eq? (.port (rtc-trigger pc)) reply-port)
+                        (not (and=> (as trigger <q-trigger>) .modeling?)))))
+           (let* ((error (make <deadlock-error> #:ast o #:message "deadlock"))
+                  (pc (clone pc #:status error)))
              (list pc)))
           (else
            (list pc)))))
@@ -339,9 +423,10 @@
          (pc (pop-pc pc))
          (other-instance
           other-port
-          (if (is-a? instance <runtime:port> )
+          (if (is-a? instance <runtime:port>)
               (runtime:other-instance+port instance)
-              (runtime:other-instance+port instance (runtime:port instance (.port q-trigger))))))
+              (runtime:other-instance+port
+               instance (runtime:port instance (.port q-trigger))))))
     (cond
      ((and (is-a? other-instance <runtime:port>)
            (ast:requires? (.ast other-instance)))
@@ -362,86 +447,60 @@
 
 (define-method (step (pc <program-counter>) (o <block>))
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
-  (if (not (q-empty? pc)) (let ((pc (set-handling? pc #f)))
+  (if (not (q-empty? pc)) (let ((pc (set-handling! pc)))
                             (list (flush pc)))
       (let ((pc (continuation pc o))
             (trigger (.trigger pc)))
         (if (ast:requires? trigger) (list pc)
             (let* ((locals (filter (is? <variable>) (ast:statement* (.parent o))))
-                   (pc (pop-locals pc locals)))
+                   (pc (pop-locals pc locals))
+                   (instance (.instance pc))
+                   (r:port (runtime:port instance (.port trigger))))
               (cond
-               ((.released pc)
-                (let ((pc (clone pc #:released #f)))
+               ((memq r:port (.released pc))
+                (let ((pc (clone pc #:released (delete r:port (.released pc)))))
                   (list pc)))
                (else
-                (let* ((instance (.instance pc))
-                       (r:port (runtime:port instance (.port trigger)))
-                       (pc (make <program-counter>
-                             #:async (.async pc)
-                             #:blocked (acons r:port pc (.blocked pc))
-                             #:external-q (.external-q pc)
-                             #:state (.state pc)
-                             #:trail (.trail pc))))
+                (let ((pc (block pc r:port)))
                   (list pc)))))))))
 
-(define-method (step (pc <program-counter>) (o <unblock>))
-  (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
-  (let* ((blocked (.blocked pc))
-         (r:port (.released pc))
-         (released (assoc-ref blocked r:port))
-         (instance (.instance pc))
-         (pc (clone pc
-                    #:blocked (alist-delete r:port blocked)
-                    #:instance (.instance released)
-                    #:released #f
-                    #:previous (.previous released)
-                    #:statement (.statement released)
-                    #:trigger (.trigger released))))
-    (list pc)))
-
 (define-method (step (pc <program-counter>) (o <end-of-on>))
-  ;; XXX FIXME: programmatical flush (vs: others are normalize+ast based).
-  (define (flush-other pc)
-    (let* ((instance (.instance pc))
-           (other-instance
-            (and (is-a? instance <runtime:component>)
-                 (let* ((ports (runtime:port* instance))
-                        (other-ports (filter ast:provides? ports))
-                        (other-port-instances (map (cut runtime:port instance <>) other-ports))
-                        (other-instances (map (cut runtime:other-instance+port instance <>) other-port-instances)))
-                   (find (compose pair? .q (cut get-state pc <>)) other-instances)))))
-      (and other-instance
-           (not (get-handling? pc other-instance))
-           (list (flush pc other-instance)))))
-
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
-  (or (flush-other pc)
-      (and (not (q-empty? pc))
-           (not (and (is-a? (and=> (.previous pc) .statement) <flush-return>)
-                     (eq? (.instance pc) (.instance (.previous pc)))))
-           (list (flush pc)))
-      (and (.released pc) (pair? (.blocked pc))
-           (let* ((locals (filter (is? <variable>) (ast:statement* (.parent o))))
-                  (pc (pop-locals pc locals)))
-             (step pc (make <unblock>))))
-      (let* ((pc (set-handling? pc #f))
-             (trigger (.trigger pc)))
-        (cond
-         ((.status pc)
-          (list (pop-pc pc)))
-         ((ast:out? trigger)
-          (let* ((locals (filter (is? <variable>) (ast:statement* (.parent o))))
-                 (pc (pop-locals pc locals)))
-            (list (pop-pc pc))))
-         (else
-          (let* ((value (->sexp (get-reply pc)))
-                 (value (or (and (not (equal? value "void")) value) "return"))
-                 (return (make <trigger-return>
-                           #:location (.location o)
-                           #:port.name (.port.name (.trigger pc))
-                           #:event.name value))
-                 (pc (clone pc #:statement (clone return #:parent (.parent o)))))
-            (list pc)))))))
+  (let* ((deferred (pop-deferred pc))
+         (pc (reset-handling! pc))
+         (trigger (.trigger pc))
+         (previous (.previous pc))
+         (instance (.instance pc))
+         (stack (pc->stack pc))
+         (stack (drop-right stack 1))
+         (queue-lengths (map (compose length .q (cute get-state <> instance))
+                             stack))
+         (livelock? (not (apply <= queue-lengths))))
+    (cond
+     ((and (is-a? instance <runtime:component>)
+           (or (and (not (q-empty? pc))
+                    (not livelock?))
+               (and (not (q-empty? pc deferred))
+                    (not (get-handling pc deferred)))))
+      (list (flush pc)))
+     ((.status pc)
+      (list (pop-pc pc)))
+     ((ast:out? trigger)
+      (let* ((locals (filter (is? <variable>) (ast:statement* (.parent o))))
+             (pc (pop-locals pc locals)))
+        (list (pop-pc pc))))
+     (else
+      (let* ((port-name (or (.port.name (.trigger pc))
+                            (.name (.ast instance))))
+             (value (->sexp (get-reply pc port-name)))
+             (value (or (and (not (equal? value "void")) value) "return"))
+             (return (make <trigger-return>
+                       #:location (.location o)
+                       #:port.name (.port.name (.trigger pc))
+                       #:event.name value))
+             (return (clone return #:parent (.parent o)))
+             (pc (clone pc #:statement return)))
+        (list pc))))))
 
 (define-method (step (pc <program-counter>) (o <trigger-return>))
   (define (valued-reply? pc o)
@@ -461,7 +520,10 @@
            (let* ((trigger (.trigger pc))
                   (type (ast:type trigger))
                   (typed? (ast:typed? type))
-                  (value (get-reply pc)))
+                  (port-name (or (.port.name o)
+                                 (.port.name (.trigger pc))
+                                 (.name (.ast instance))))
+                  (value (get-reply pc port-name)))
              (and typed? (not value))))))
 
   (%debug "  ~s ~s ~a\n" ((compose name .instance) pc) (and=> (.trigger pc) trigger->string) (name o))
@@ -470,7 +532,8 @@
     (let* ((trigger (.trigger pc))
            (type (ast:type trigger))
            (typed? (ast:typed? type))
-           (error (make <missing-reply-error> #:ast o #:type type #:message "missing-reply")))
+           (error (make <missing-reply-error> #:ast o #:type type
+                        #:message "missing-reply")))
       (%debug "missing reply\n")
       (list (clone pc #:status error))))
    (else
@@ -480,7 +543,43 @@
            (pc (if (or blocking? (.status pc)) pc
                    (let ((locals (filter (is? <variable>)
                                          (ast:statement* (.parent o)))))
-                     (pop-locals pc locals)))))
+                     (pop-locals pc locals))))
+           (port (.port o))
+           (instance (.instance pc))
+           (value (and port (get-reply pc (.name port))))
+           (pc (cond
+                ((and (not blocking?)
+                      (is-a? (ast:type value) <void>))
+                 (let ((error (make <second-reply-error>
+                                #:ast o
+                                #:previous (.parent value)
+                                #:message "second-reply")))
+                   (%debug "second reply, previous=~a\n" (ast:location->string value))
+                   (clone pc #:status error)))
+                ((not blocking?)
+                 pc)
+                (else
+                 (let* ((r:port (runtime:port instance port))
+                        (blocked (.blocked pc))
+                        (blocked? (assq-ref blocked r:port))
+                        (blocked (if (not blocked?) blocked
+                                     (alist-delete r:port blocked)))
+                        (collateral (.collateral pc))
+                        (collateral-released (.collateral-released pc))
+                        (collateral-released
+                         (if (not (assq-ref collateral r:port)) collateral-released
+                             (append collateral-released (list r:port))))
+                        (released (.released pc))
+                        (released (delete r:port released)))
+                   (clone pc
+                          #:blocked blocked
+                          #:released released
+                          #:collateral-released collateral-released)))))
+           (reset-blocked-on-boundary?
+            (and (blocked-on-boundary? pc)
+                 (eq? instance (blocked-on-boundary? pc))))
+           (pc (if reset-blocked-on-boundary? (blocked-on-boundary-reset pc)
+                   pc)))
       (list (pop-pc pc))))))
 
 (define-method (step (pc <program-counter>) (o <flush-async>))
